#!/usr/bin/perl
#
# Id     : tinydns-stats.pl
# Author : Jay Soffian <jay@soffian.org>
# Purpose: Generate report from tinydns query log
# Usage  : tai64nlocal < logfile | tinydns-stats.pl
# History: 2001-04-01: initial (based off a formater I previously 
#          wrote to grok bind query logs)
#

use strict;
use Socket;

main();
exit 0;

sub main {

  my %q_types = (
   0x01 => "A",
   0x02 => "NS",
   0x05 => "CNAME",
   0x06 => "SOA",
   0x0C => "PTR",
   0x0D => "HINFO",
   0x0F => "MX",
   0x10 => "TXT",
   0x11 => "RP",
   0x18 => "SIG",
   0x19 => "KEY",
   0x1C => "AAAA",
   0xFC => "AXFR",
   0xFF => "ANY",
  );

  my ($total, @hour, %src_ip, %q, %q_type, $first, $last);

  while(<>) {
    chomp;
    my ($date, $time, $src, $flag, $q_type, $q) = split;
    my ($src_ip, $src_port, $q_id) = split (/:/,$src);
    $src_ip = join(".", unpack("C*", pack("H8", $src_ip)));
    $src_port = hex $src_port;
    $q_id = hex ($q_id);
    $q_type = ($q_types{hex($q_type)} || '0x' . $q_type);

    next unless $date;
    $first ||= "$date $time";
    $last = "$date $time";

    my ($hour) = $time =~ /^(\d{2}):/;
    $q =~ tr[A-Z][a-z]; # normalize
    $total++;
    $hour[$hour]++;
    $src_ip{$src_ip}++;
    $q{$q}->{total}++;
    $q{$q}->{type}->{$q_type}++;
    $q_type{$q_type}++;
  }
  print "DNS statistics for $first - $last\n\n";
  total_requests($total);
  requests_by_type(\%q_type);
  requests_by_hour(\@hour);
  topN_requests(\%q,150);
  topN_requesters(\%src_ip,150);
}

sub total_requests {
  my ($total) = @_;
  print "Total requests: $total\n";
  print "\n";

}
sub requests_by_type{
  my ($hr) = @_;
  print "Requests by type:\n";
  foreach (sort {$hr->{$b} <=> $hr->{$a}} keys %$hr) {
    printf "  %-6s %d\n",$_, $hr->{$_};
  }
  print "\n";

}
sub requests_by_hour{
  my ($ar) = @_;
  print "Requests by hour:\n";
  my $max_by_hour = 1;
  for (my $i=0;$i<24;$i++) {
    $max_by_hour = $ar->[$i] if $ar->[$i] > $max_by_hour;
  }
  for (my $i=0;$i<24;$i++) {
    my $s = '*' x ($ar->[$i] / $max_by_hour * 60);
    printf "  %02d %-8d %s\n",$i,$ar->[$i],$s;
  }
  print "\n";
}

sub topN_requests{
  my ($hr, $n) = @_;
  print "Top $n Requests:\n";
  my $i=0;
  foreach (sort {$hr->{$b}->{total} <=> $hr->{$a}->{total}} keys %$hr) {
    last if ++$i > $n;
    printf "  %-6d %-40s ",$hr->{$_}->{total},$_;
    foreach my $q_type (sort { $hr->{$_}->{type}->{$b} <=> 
				$hr->{$_}->{type}->{$a}} keys %{$hr->{$_}->{type}}) {
      printf "%s:%s ", $q_type, $hr->{$_}->{type}->{$q_type};
    }
    print "\n";
  }
  print "\n";
}
sub topN_requesters{
  my ($hr, $n) = @_;
  print "Top $n Requesters:\n";
  my $i=0;
  foreach (sort {$hr->{$b} <=> $hr->{$a}} keys %$hr) {
    last if ++$i > $n;
    printf "  %-6d %-16s %s\n",$hr->{$_},$_,gethostbyaddr(inet_aton($_), AF_INET);
  }
  print "\n";
}


